/********************************************************************
 * Description:
 * The server code which will wait for a client request and then set
 * up a socket through which data is written, read or erassed.
 ********************************************************************/

// Generic Header files containing function declarations 
// and global declarations

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <sys/ioctl.h>
#include "disk_app.h"

#define VERSION_NUM "v_0.30"
void fd_read(void);
void fd_write(void);
void process_message1(blk_drv_if *sock_fd, int command);
void ack_cmd_done(void);


#define LBA_SIZE_IN_BYTES		  (512)
#define DISK_SIZE             (10*1024*LBA_SIZE_IN_BYTES*sizeof(char))

enum cmd_type_t 
{ 
	LBA_WRITE,
	LBA_READ,
	LBA_ERASE,
	LBA_EXIT,
	LBA_NO_SUCH_FUNC
};

enum cmd_type_t comm;
int    fd;  /* input sources 1 */
fd_set readfs;    /* file descriptor set */
blk_drv_if tmp_msg_buffer;
blk_drv_if fd_msg;
char *m_disk;


int main(int argc, char *argv[])
{       
  m_disk = NULL;

	if (init_blk_device_if()){
		return -1;
	}

  m_disk = malloc(DISK_SIZE);
  if (m_disk == NULL) {
    printf("Failed to get mdisk buffer \n");
  } else {
    printf("mdisk buffer ready\n");
  }
 
	while(1) {
		get_blk_device_msg();
		comm = tmp_msg_buffer.command;
		if (comm == LBA_READ) {
			fd_read();
		} else {
			fd_write(); 
		}
    ack_cmd_done();
	}
 
}

void ack_cmd_done(void)
{
  int tmp1 = 0;
  int rc = 0;
  rc = ioctl(fd, FD_DONE, &tmp1);
}

void fd_read(void)
{
  int lba_num;
  int ReqNumBytes; 
  int offset; 

	ReqNumBytes = tmp_msg_buffer.data_length;
	lba_num = tmp_msg_buffer.blk_num;
  offset =  lba_num*512; 

  printf("lba_read(): @ %d, %d bytes\n", lba_num, ReqNumBytes);
  memcpy(tmp_msg_buffer.data, (m_disk+offset), ReqNumBytes);

  write(fd, &tmp_msg_buffer, sizeof(tmp_msg_buffer));
}

void fd_write(void)
{
  int lba_num;
  int ReqNumBytes; 
  int offset; 
  offset =  lba_num*512; 

	ReqNumBytes = tmp_msg_buffer.data_length;
	lba_num = tmp_msg_buffer.blk_num;

  printf("lba_write(): @ %d, %d bytes\n", lba_num, ReqNumBytes);
  memcpy((m_disk+offset), tmp_msg_buffer.data, tmp_msg_buffer.data_length);
}

void get_blk_device_msg(void)
{
	/* block until input becomes available */
	select((fd+1), &readfs, NULL, NULL, NULL);
	if (FD_ISSET(fd, &readfs)) {
		read(fd, &tmp_msg_buffer, sizeof(blk_drv_if));
	} else {
		printf("Error with Select... \n");
	}
	
}

int init_blk_device_if(void)
{
  fd = open("/dev/char_drv", O_RDWR);
  FD_SET(fd, &readfs); 
	if (fd<0) {
		printf("Err: Unable to open file\n"); 
		return -1;
	}
	return 0;
}

void displayMemory(unsigned char *data, int length) 
{
	int i = 0; //used to keep track of line lengths
	unsigned char ch; // also used to print char version of data
	
	printf("===============\n");
	for (i=0; i <length; i++) {
		if ((i%16)== 0) {
			printf("\n");
		}
    ch = data[i];
    if ((ch < 33) || (ch > 126)) {
      printf("0x%02x ", ch);
    } else {
      printf("   %c ", ch);
    }
	}
	printf("\n===============\n\n\n");
}

void dumpMemory(unsigned char *data, int length) 
{
	int i = 0; //used to keep track of line lengths
	unsigned char ch; // also used to print char version of data
	
	for (i=0; i <length; i++) {
		if ((i%16)== 0) {
      printf("\n%04d::", i);
		}
    ch = data[i];
    printf("0x%02x ", ch);
	}
	printf("\n===============\n\n\n");
}

